'From Haver 6.0 [latest update: #5092] on 9 March 2022 at 9:50:03 pm'!
'Description In memory or in-image transactions.'!
!provides: 'Transactions' 1 26!
!requires: 'WriteBarrier' 1 23 nil!
SystemOrganization addCategory: 'Transactions'!


!moduleCreation: Transactions!
Modules newEnvironment: #Transactions!

!interfacesOf: Transactions!
Modules environment: #Transactions ::
	interface: #SPI exporting: #(#TransactiveObject #TransactionManager #Transaction #ObjectTracker #TransactionLeader ) ::
	interface: #UTI exporting: #(#ModificationForbiddenHandler ) ::
	interface: #API exporting: #(#TransactiveObject #TransactionManager #TransactionLeader )!


!classDefinition: (Modules>>#Transactions>>#ModificationForbiddenHandler) category: 'Transactions'!
Object subclass: #ModificationForbiddenHandler
	instanceVariableNames: 'transactionManagers oldModificationHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transactions'
	inModule: #Transactions!
!classDefinition: (Modules>>#Transactions>>#ModificationForbiddenHandler) class category: 'Transactions'!
(Modules>>#Transactions>>#ModificationForbiddenHandler) class
	instanceVariableNames: 'uniqueInstance lock'!

!classDefinition: (Modules>>#Transactions>>#ObjectTracker) category: 'Transactions'!
Object subclass: #ObjectTracker
	instanceVariableNames: 'involvedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transactions'
	inModule: #Transactions!
!classDefinition: (Modules>>#Transactions>>#ObjectTracker) class category: 'Transactions'!
(Modules>>#Transactions>>#ObjectTracker) class
	instanceVariableNames: ''!

!classDefinition: (Modules>>#Transactions>>#Transaction) category: 'Transactions'!
(Modules>>#Transactions>>#ObjectTracker) subclass: #Transaction
	instanceVariableNames: 'id changedObjects savedObjects state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transactions'
	inModule: #Transactions!
!classDefinition: (Modules>>#Transactions>>#Transaction) class category: 'Transactions'!
(Modules>>#Transactions>>#Transaction) class
	instanceVariableNames: ''!

!classDefinition: (Modules>>#Transactions>>#TransactionManager) category: 'Transactions'!
(Modules>>#Transactions>>#ObjectTracker) subclass: #TransactionManager
	instanceVariableNames: 'transactions transactionsByProcess modifyingTransactionByObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transactions'
	inModule: #Transactions!
!classDefinition: (Modules>>#Transactions>>#TransactionManager) class category: 'Transactions'!
(Modules>>#Transactions>>#TransactionManager) class
	instanceVariableNames: 'uniqueInstance lock'!

!classDefinition: (Modules>>#Transactions>>#TransactiveObject) category: 'Transactions'!
Object subclass: #TransactiveObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transactions'
	inModule: #Transactions!
!classDefinition: (Modules>>#Transactions>>#TransactiveObject) class category: 'Transactions'!
(Modules>>#Transactions>>#TransactiveObject) class
	instanceVariableNames: ''!

!classDefinition: (Modules>>#Transactions>>#TransactionLeader) category: 'Transactions'!
(Modules>>#Transactions>>#TransactiveObject) subclass: #TransactionLeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transactions'
	inModule: #Transactions!
!classDefinition: (Modules>>#Transactions>>#TransactionLeader) class category: 'Transactions'!
(Modules>>#Transactions>>#TransactionLeader) class
	instanceVariableNames: ''!


!(Modules>>#Transactions>>#ModificationForbiddenHandler) commentStamp: '<historical>' prior: 0!
I coordinate between all the transaction managers registered with me.!

!(Modules>>#Transactions>>#ObjectTracker) commentStamp: '<historical>' prior: 0!
I am an abstract super class that tracks object.!

!(Modules>>#Transactions>>#Transaction) commentStamp: '<historical>' prior: 0!
I am a transaction.!

!(Modules>>#Transactions>>#TransactionManager) commentStamp: '<historical>' prior: 0!
I manage all transactions.

`transactions` is an identity set of all know transactions. This inlcudes aborted transactions.
`transactionsByObject` is a weak key identity dictionary that maps objects to their transactions ,if any.,
`transactionsByProcess` is a weak key identity dictionary that maps processes to their
current process, if any. During the livetime of process the mapping may change, even
in a stacklike manner.!

!(Modules>>#Transactions>>#TransactiveObject) commentStamp: '<historical>' prior: 0!
I support being part of a transaction. Especially I am able to be a transaction leader.!

!(Modules>>#Transactions>>#TransactionLeader) commentStamp: '<historical>' prior: 0!
I am a transaction leader.!

!(Modules>>#Transactions>>#ModificationForbiddenHandler) methodsFor: 'initialization' stamp: 'KLG 3/9/2022 20:47:33'!
initialize
	"Initialize my set of transaction managers."

	super initialize.
	transactionManagers _ WeakIdentitySet new.
	self installMyGlobalBarrierHandler! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) methodsFor: 'registration' stamp: 'KLG 2/26/2022 16:16:21'!
register: aTransactionManager
	"Register aTransactionManager with me"

	transactionManagers add: aTransactionManager! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) methodsFor: 'registration' stamp: 'KLG 2/26/2022 16:16:48'!
unRegister: aTransactionManager
	"unRegister aTransactionManager with me"

	transactionManagers remove: aTransactionManager ifAbsent: []! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) methodsFor: 'writebarrier' stamp: 'KLG 3/9/2022 20:44:42'!
installMyGlobalBarrierHandler
	"Install code a global write barrier handler.."

	self assert: oldModificationHandler isNil.
	oldModificationHandler _ ModificationForbidden 
		handleModificationForbidden: self with: #objectModified:collectionIndex:newValue:selector:! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) methodsFor: 'writebarrier' stamp: 'KLG 3/9/2022 21:29:15'!
objectModified: anObject collectionIndex: anIndex newValue: newValue selector: aSelector
	"""Handle a write barrier for an object."""
	
	| handlingTransactionManagers |
	handlingTransactionManagers _ transactionManagers select: [ :manager |
		manager handlesModificationForbiddenFor: anObject ].
	handlingTransactionManagers ifEmpty: [ ^ false ].
	handlingTransactionManagers size > 1 ifTrue: [
		self flagHaver: 'This might be problematic once we introduce subtransactions'.
		self error: 'Object handled by more than one transaction manager' ].
	^ handlingTransactionManagers anyOne 
		objectModified: anObject
		collectionIndex: anIndex
		newValue: newValue
		selector: aSelector
			! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) methodsFor: 'writebarrier' stamp: 'KLG 3/9/2022 20:45:19'!
uninstallMyGlobalBarrierHandler
	"Uninstall code a global write barrier handler.."

	ModificationForbidden modificationForbiddenHandler: oldModificationHandler.
	oldModificationHandler _ nil! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) class methodsFor: 'singleton' stamp: 'KLG 3/9/2022 20:39:42'!
uniqueInstance
	"comment stating purpose of message"

	^ uniqueInstance! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) class methodsFor: 'class initialization' stamp: 'KLG 3/9/2022 20:39:29'!
initialize
	"Initialize my single TransactionManager."
	
	uniqueInstance _ self new.
	lock _ Mutex new! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) class methodsFor: 'registration' stamp: 'KLG 3/9/2022 20:40:29'!
register: aTransactionManager
	"Register aTransactionManager with me"

	lock critical: [ uniqueInstance register: aTransactionManager ]! !

!(Modules>>#Transactions>>#ModificationForbiddenHandler) class methodsFor: 'registration' stamp: 'KLG 3/9/2022 20:41:10'!
unRegister: aTransactionManager
	"unRegister aTransactionManager with me"

	lock critical: [ uniqueInstance unRegister: aTransactionManager ]! !

!(Modules>>#Transactions>>#ObjectTracker) methodsFor: 'initialization' stamp: 'KLG 1/26/2019 14:16:53'!
initialize
	"Initialize the set of involved objects."

	super initialize.
	involvedObjects _ WeakIdentitySet new! !

!(Modules>>#Transactions>>#ObjectTracker) methodsFor: 'transactions - private' stamp: 'KLG 1/31/2019 18:24:46'!
register: anObject
	"Register anObject with this transaction."

	involvedObjects add: anObject.
! !

!(Modules>>#Transactions>>#ObjectTracker) methodsFor: 'transactions - private' stamp: 'KLG 6/19/2021 18:03:56'!
registerReferencedObjectsOf: anObject
	"Register all objects that are referenced by anObject."

	| class index |
	class _ anObject class.
	class isVariable ifTrue: [
		index _ anObject basicSize.
		[ index > 0 ] whileTrue: [
			self registerTransitivly: (anObject basicAt: index).
			index _ index - 1 ]].
	index _ class instSize.
	[ index > 0 ] whileTrue: [
		self registerTransitivly: (anObject instVarAt: index).
		index _ index - 1 ].! !

!(Modules>>#Transactions>>#ObjectTracker) methodsFor: 'transactions - private' stamp: 'KLG 6/19/2021 18:03:33'!
registerTransitivly: anObject
	"Register anObject and all the objects referenced by it.
	
	Answer true if the object was registered."

	anObject ifNil: [ ^ false ].
	involvedObjects includes: anObject :: ifTrue: [ ^ false ].
	involvedObjects add: anObject.
	anObject registerMyReferencedObjectsWith: self.
	^ true! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'initialization' stamp: 'KLG 1/26/2019 15:29:52'!
initialize
	"Initialize the set of changed objects."

	super initialize.
	state _ #new.
	changedObjects _ WeakIdentitySet new.
	savedObjects _ WeakIdentityKeyDictionary new! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'accessing' stamp: 'KLG 1/26/2019 14:19:03'!
changedObjects
	"Answer the value of changedObjects"

	^ changedObjects! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'accessing' stamp: 'KLG 1/26/2019 14:20:28'!
id
	"Answer the value of id"

	^ id ifNil: [ id _ self class name, self identityHash asString ]! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'accessing' stamp: 'KLG 1/26/2019 14:19:41'!
id: anObject
	"Set the value of id"
	
	self assert: id isNil.
	id _ anObject! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 2/25/2022 15:09:24'!
abort
	"Mark the rollback of a transaction."

	self assert: state == #active.
	state _ #aborted.
	savedObjects keysDo: [ :savedObject | savedObject beWritableObjectIfSensible ].
	savedObjects keys asArray elementsForwardIdentityTo: savedObjects values asArray.
	"Make transactive objects readonly, again."
	savedObjects do: [ :object |
		object isTransactiveObject ifTrue: [ object beReadOnlyObjectIfSensible ] ].
	savedObjects _ nil.
	self makeInvolvedObjectsWriteable.
	self managerNoLocking removeTransaction: self! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 6/19/2021 18:02:31'!
begin
	"Mark the begin of a transaction."

	self assert: state == #new.
	self assert: changedObjects isEmpty.
	state _ #active.
	involvedObjects do: [ :involvedObject |
		involvedObject registerMyReferencedObjectsWith: self ].
	involvedObjects do: [ :involvedObject |
		involvedObject beReadOnlyObjectIfSensible ]
	! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 2/25/2022 11:26:32'!
commit
	"Commit this transaction (me)."

	self assert: state == #active.
	state _ #commited.
	self makeInvolvedObjectsWriteable.
	self managerNoLocking
		committed: self 
		withInvolvedObjects: involvedObjects.
	involvedObjects _ savedObjects _ nil
	! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 1/30/2019 20:02:29'!
isRegistered: anObject
	"Answer true if anObject si registered with this transaction."
	
	^ involvedObjects includes: anObject ! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 2/25/2022 11:27:02'!
makeInvolvedObjectsWriteable
	"Make all the involved objects writeable (again)."

	involvedObjects do: [ :involvedObject |
		involvedObject isTransactiveObject ifFalse: [
			involvedObject beWritableObjectIfSensible ] ]! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 5/21/2021 16:53:53'!
manager

	^ TransactionManager! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 5/21/2021 16:53:46'!
managerNoLocking

	^ TransactionManager uniqueInstance! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions' stamp: 'KLG 6/19/2021 18:02:59'!
register: anObject
	"Register anObject with this transaction."

	| answer |
	self assert: #state ~~ #aborted.
	answer _ super register: anObject.
	state == #active ifTrue: [ 
		anObject beReadOnlyObjectIfSensible.
		anObject registerMyReferencedObjectsWith: self ].
	^ answer! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions - private' stamp: 'KLG 6/19/2021 22:01:48'!
registerObjectForModification: anObject
	"Register anObject with this transaction."

	state == #active ifFalse: [
		^ self ].
	self manager registerObjectForModification: anObject by: self ::
		ifFalse: [ self error: 'Transaction conflict' ].  ":: Do better than that!!."
	savedObjects at: anObject put: anObject shallowCopy.
	anObject beReadOnlyObjectIfSensible! !

!(Modules>>#Transactions>>#Transaction) methodsFor: 'transactions - private' stamp: 'KLG 5/21/2021 16:54:19'!
registerTransitivly: anObject
	"Register anObject and all the objects referenced by it."

	| answer |
	(answer _ super registerTransitivly: anObject) ifTrue: [
		self managerNoLocking ":: We already have the lock, take a shortcut here."
			registerReferencedObject: anObject.	
		state == #active ifTrue: [ anObject  beReadOnlyObjectIfSensible ] ].
	^ answer! !

!(Modules>>#Transactions>>#Transaction) class methodsFor: 'instance creation' stamp: 'KLG 1/26/2019 14:21:35'!
withId: anId
	"Answer a new transaction identified by anId."

	^ self new;
		id: anId;
		yourself ! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'initialization' stamp: 'KLG 3/9/2022 20:47:14'!
initialize
	"Initialize the set of changed objects."

	super initialize.
	transactions _ IdentitySet new.
	modifyingTransactionByObject _ WeakIdentityKeyDictionary new.
	transactionsByProcess _ WeakIdentityKeyDictionary new.
	ModificationForbiddenHandler register: self! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 1/31/2019 19:10:52'!
abort
	"Abort the current transaction."

	^ self active abort! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 5/21/2021 16:48:48'!
active
	"Answer the transaction for this process."

	| currentProcess newTransaction |
	^ transactionsByProcess at: (currentProcess _ Processor activeProcess) ifAbsent: [
			transactionsByProcess 
				at: currentProcess 
				put: (newTransaction _ self newTransaction).
			transactions add: newTransaction ]! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 1/31/2019 19:07:53'!
begin
	"Begin the active transaction."

	^ self active begin! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 1/3/2021 22:49:44'!
begin: anId
	"Begin the active transaction give it an id."

	^ self active id: anId :: begin! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 1/31/2019 19:11:08'!
commit
	"Commit the current transaction."

	^ self active commit! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 1/29/2019 15:44:39'!
do: aBlock inMyTransaction: anObject
	"Evaluate a block while anObject transaction is the active one for this process."
	
	| currentProcess |
	anObject 
		at: (currentProcess _ Processor activeProcess)
		ifPresent: [ :oldTransaction |
			transactionsByProcess 
				at: currentProcess 
				put: (self register: anObject).
			aBlock ensure: [
				transactionsByProcess 
					at: currentProcess 
					put: oldTransaction ]]
		ifAbsent: [ | transactionToRemove |
			transactionsByProcess 
				at: currentProcess 
				put: (transactionToRemove _ self register: anObject).
			aBlock ensure: [ transactionsByProcess removeKey: transactionToRemove ]].! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 1/30/2019 20:12:28'!
isRegistered: anObject 
	"Answer if anObject is registered."
	
	^ involvedObjects includes: anObject
	! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions' stamp: 'KLG 1/31/2019 19:31:52'!
register: anObject
	"Register anObject and answer its transaction."

	super register: anObject.
	^ self active register: anObject; yourself ! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'instance creation' stamp: 'KLG 5/21/2021 16:48:48'!
newTransaction

	^ self transactionClass new! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'instance creation' stamp: 'KLG 1/26/2019 14:31:14'!
transactionClass
	"Answer the transaction class for me."

	^ self class transactionClass! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions - private' stamp: 'KLG 2/25/2022 15:00:59'!
committed: aTransaction withInvolvedObjects: someInvolvedObjects 
	"aTransaction was commited with its involved objects."
	
	modifyingTransactionByObject keysAndValuesRemove: [ :object :transaction |
		aTransaction == transaction and: [ someInvolvedObjects includes: object ] ].
	self removeTransaction: aTransaction ! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions - private' stamp: 'KLG 2/5/2019 22:31:31'!
registerObjectForModification: anObject
	"Register anObject for modification."

	self active registerObjectForModification: anObject! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions - private' stamp: 'KLG 1/28/2019 21:30:33'!
registerObjectForModification: anObject by: aTransaction
	"Register anObject as changed by a transaction.
	
	Answer true if the object is not modied by another transaction."
	
	^ modifyingTransactionByObject
		at: anObject
		ifPresent: [ :transaction |
			transaction == aTransaction ]
		ifAbsent: [
			modifyingTransactionByObject
				at: anObject
				put: aTransaction.
			true ].! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions - private' stamp: 'KLG 2/5/2019 22:27:42'!
registerReferencedObject: anObject
	"Register an object that was transitivly referenced by some transaction.."

	super register: anObject! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'transactions - private' stamp: 'KLG 2/25/2022 15:00:59'!
removeTransaction: aTransaction
	"Remove aTransaction from my collections of transactions."

	transactionsByProcess removeKey: Processor activeProcess.
	transactions remove: aTransaction! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'writebarrier' stamp: 'KLG 2/26/2022 16:08:00'!
handlesModificationForbiddenFor: anObject
	"Answer true when I handle a ModificationForbidden exception for anObject."
 
	^ self isRegistered: anObject! !

!(Modules>>#Transactions>>#TransactionManager) methodsFor: 'writebarrier' stamp: 'KLG 2/7/2019 04:12:32'!
objectModified: anObject collectionIndex: anIndex newValue: newValue selector: aSelector
	"""Handle a write barrier for an object."""
	
	self isRegistered: anObject :: ifFalse: [ ^ false ].
	self 
		register: newValue;
		registerObjectForModification: anObject.
	anObject 
		beWritableObjectIfSensible;
		perform: aSelector with: anIndex with:  newValue;
		beReadOnlyObjectIfSensible.
	^ true
			! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'class initialization' stamp: 'KLG 5/20/2021 17:41:25'!
initialize
	"Initialize my single TransactionManager."
	
	uniqueInstance _ self new.
	lock _ Mutex new! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'singleton' stamp: 'KLG 5/20/2021 17:41:32'!
uniqueInstance
	"Answer the single transaction manager."

	^ uniqueInstance! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'instance creation' stamp: 'KLG 1/26/2019 14:31:36'!
transactionClass
	"Answer the transaction class for me."

	^ Transaction ! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:10'!
abort
	"Abort the current transaction."

	^ lock critical: [ self uniqueInstance abort ]! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:16'!
active
	"Answer the active transaction for this process."

	^ lock critical: [ self uniqueInstance active ]! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:20'!
begin
	"Begin the active transaction."

	^ lock critical: [ self uniqueInstance begin ]! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:26'!
commit
	"Commit the current transaction."

	^ lock critical: [ self uniqueInstance commit ]! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:31'!
do: aBlock inMyTransaction: anObject
	"Evaluate a block while anObject transaction is the active one for this process."
	
	^ lock critical: [ self uniqueInstance do: aBlock inMyTransaction: anObject ]! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:35'!
isRegistered: anObject
	"Answer true if anObject is registered."
	
	^ lock critical: [ self uniqueInstance isRegistered: anObject ]
	! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:40'!
register: anObject
	"Register anObject with me and answer the transaction for anObject"

	^ lock critical: [ self uniqueInstance register: anObject ]! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'transactions' stamp: 'KLG 5/20/2021 17:42:48'!
registerObjectForModification: anObject by: aTransacrion
	"Register anObject as changed by a transaction.
	
	Answer true if the object is not modied by another transaction."

	^ lock critical: [ 
		self uniqueInstance registerObjectForModification: anObject by: aTransacrion ]! !

!(Modules>>#Transactions>>#TransactionManager) class methodsFor: 'writebarrier' stamp: 'KLG 5/20/2021 17:43:06'!
objectModified: anObject collectionIndex:  anIndex newValue: newValue selector: aSelector
	"Handle a write barrier hit."
	
	^ lock critical: [ 
		self uniqueInstance 
			objectModified: anObject 
			collectionIndex:  anIndex 
			newValue: newValue
			selector: aSelector ]! !

!(Modules>>#Transactions>>#TransactiveObject) methodsFor: 'transactions' stamp: 'KLG 1/3/2021 21:06:34'!
registerWithTransaction
	"Register me with the current transaction."
	
	TransactionManager register: self! !

!(Modules>>#Transactions>>#TransactiveObject) methodsFor: 'initialization' stamp: 'KLG 1/26/2019 14:58:51'!
initialize
	"Register me with the my transaction amanager."

	super initialize.
	self registerWithTransaction! !

!(Modules>>#Transactions>>#TransactiveObject) methodsFor: 'testing' stamp: 'KLG 1/3/2021 21:04:19'!
is: aSymbol
	"Support the common catgeorisation stuff."

	^ aSymbol == #TransactiveObject or: [ super is: aSymbol ]! !

!(Modules>>#Transactions>>#TransactiveObject) methodsFor: 'testing' stamp: 'KLG 2/9/2019 19:36:04'!
isTransactiveObject
	"Answer true, becuase I am a transactive object."
	
	^ true! !

!Object methodsFor: '*Transactions' stamp: 'KLG 2/9/2019 19:36:55'!
isTransactiveObject
	"Answer false, because in general objects are not transactive."
	
	^ false! !

!Object methodsFor: '*Transactions' stamp: 'KLG 6/19/2021 17:59:01'!
registerMyReferencedObjectsWith: aTransaction
	"Register my my referenced objects -- the objects in my instance variables - 
	with aTransaction."
	
	^ aTransaction registerReferencedObjectsOf: self! !

!Behavior methodsFor: '*Transactions' stamp: 'KLG 6/19/2021 18:00:49'!
registerMyReferencedObjectsWith: aTransaction
	"Register my my referenced objects -- the objects in my instance variables - 
	with aTransaction.
	
	Do nothing here, we dont care for classes."! !

!CompiledMethod methodsFor: '*Transactions' stamp: 'KLG 6/19/2021 18:01:34'!
registerMyReferencedObjectsWith: aTransaction
	"Register my my referenced objects -- the objects in my instance variables - 
	with aTransaction.
	
	Do nothing here, we dont care for compiled methods."! !
(Modules>>#Transactions>>#ModificationForbiddenHandler) initialize!
(Modules>>#Transactions>>#TransactionManager) initialize!
