'From Haver 6.0 [latest update: #5065] on 9 February 2022 at 10:35:47 am'!
'Description Provides keyboard focus indication in the title-bar of SystemWindows'!
!provides: 'KeyboardFocus' 1 5!



!Preferences class methodsFor: '*KeyboardFocus-standard queries' stamp: 'KLG 11/29/2021 12:44:29'!
titlebarIndicatesKeyboardFocus
	"Answer true if the title bar should indicate the keyboard focus."

	^ self
		valueOfFlag: #titlebarIndicatesKeyboardFocus
		ifAbsent: [ true ].! !

!Morph methodsFor: '*KeyboardFocus-focus handling' stamp: 'KLG 12/2/2021 16:18:50'!
catchesKeyboardFocus
	"Answer true if I catch the keyboard focus, if no one else wants it.
	
	Morphs that are containes that handle events for the whole assembly,
	should answer true.
	
	Needed for SystemWindows to always enable Ctrl/Cmd-W to close the window
	and such."

	^ false! !

!Morph methodsFor: '*KeyboardFocus-events-processing' stamp: 'KLG 12/2/2021 19:28:19'!
processMouseEnter: aMouseEvent localPosition: localEventPosition
	"System level event handling."

	aMouseEvent isDraggingEvent ifTrue: [
		^self].
	self wantsBalloon ifTrue: [
		aMouseEvent hand triggerBalloonFor: self after: self balloonHelpDelayTime ].
	self handlesMouseOver: aMouseEvent ::
		ifTrue: [
			aMouseEvent wasHandled: true.
			self mouseEnter: aMouseEvent ]
		ifFalse: [
			"Catch the keyboard focus, if we are a catcher."
			Preferences focusFollowsMouse and: [ self catchesKeyboardFocus ] ::
				ifTrue: [ aMouseEvent hand newKeyboardFocus: self ] ]			! !

!Morph methodsFor: '*KeyboardFocus-events-processing' stamp: 'KLG 12/2/2021 19:26:09'!
processMouseLeave: anEvent localPosition: localEventPosition
	"System level event handling."
	
	anEvent hand removePendingBalloonFor: self.
	anEvent isDraggingEvent ifTrue: [
		^self].
	self handlesMouseOver: anEvent ::
		ifTrue: [
			anEvent wasHandled: true.
			self mouseLeave: anEvent ]
		ifFalse: [
			"Maybe we caught the keyboard focus, release it, just in case."
			Preferences focusFollowsMouse and: [ self catchesKeyboardFocus ] ::
				ifTrue: [ anEvent hand releaseKeyboardFocus: self ] ]
! !

!HandMorph methodsFor: '*KeyboardFocus-focus handling' stamp: 'KLG 11/29/2021 12:59:33'!
newKeyboardFocus: aMorphOrNil
	"Make the given morph the new keyboard focus, canceling the previous keyboard focus if any. If the argument is nil, the current keyboard focus is cancelled."

	| oldFocus |
	oldFocus _ keyboardFocus.
	keyboardFocus _ aMorphOrNil.
	oldFocus ifNotNil: [
		oldFocus == aMorphOrNil ifFalse: [ oldFocus keyboardFocusChange: false ].
		Preferences titlebarIndicatesKeyboardFocus ifTrue: [
			oldFocus owningWindow ifNotNil: [ :window |
				window redrawNeeded ]]].
	aMorphOrNil ifNotNil: [
		aMorphOrNil keyboardFocusChange: true.
		Preferences titlebarIndicatesKeyboardFocus ifTrue: [
			aMorphOrNil owningWindow ifNotNil: [ :window |
				window redrawNeeded ]]].! !

!HandMorph methodsFor: '*KeyboardFocus-focus handling' stamp: 'KLG 12/2/2021 19:31:56'!
releaseKeyboardFocus: aMorph
	"If the given morph had the keyboard focus before, release it"

	keyboardFocus ifNotNil: [ | morphFound catcher |
		morphFound _ false.
		keyboardFocus withAllOwnersDo: [ :outerOwner |
			morphFound
				ifFalse: [
					outerOwner == aMorph ifTrue: [				morphFound _ true ] ]
				ifTrue: [
					catcher ifNil: [
						outerOwner catchesKeyboardFocus ifTrue: [
							catcher _ outerOwner ] ] ] ].
		morphFound ifTrue: [
			catcher
				ifNil: [ 	self releaseKeyboardFocus ]
				ifNotNil: [ self newKeyboardFocus: catcher ] ] ]! !

!SystemWindow methodsFor: '*KeyboardFocus-focus handling' stamp: 'KLG 12/2/2021 16:19:42'!
catchesKeyboardFocus
	"Answer true if I catch the keyboard focus, if no one else wants it.
	
	Morphs that are containes that handle events for the whole assembly,
	should answer true.
	
	Needed for SystemWindows to always enable Ctrl/Cmd-W to close the window
	and such."

	^ true! !

!SystemWindow methodsFor: '*KeyboardFocus-drawing' stamp: 'KLG 12/2/2021 14:25:33'!
drawClassicFrameOn: aCanvas color: windowFrameColor
	"Window border encompasses title area. No round corners. No title gradient."

	aCanvas
		fillRectangle: (self morphLocalBounds insetBy: 1.5)
		color: color
		borderWidth: borderWidth
		borderStyleSymbol: #simple
		baseColorForBorder: self widgetsColor.
	"A border was drawn at the left, top and right of the title area.
	The look is that the title area is inside the window"
	aCanvas
		fillRectangle: (borderWidth @ borderWidth extent: extent x - (2 * borderWidth) @ self labelHeight)
		color:
			(self submorphHasKeyboardFocus 
				ifTrue: [ Theme current titlebarColorFromWindowFrameColor: windowFrameColor ]
				ifFalse: [ windowFrameColor ]).! !

!SystemWindow methodsFor: '*KeyboardFocus-drawing' stamp: 'KLG 12/2/2021 14:26:01'!
drawRoundedFrameOn: aCanvas color: widgetsColor

	aCanvas
		drawRoundedFrame: self morphLocalBounds
		border: borderWidth 
		color: widgetsColor
		insideColor: color
		labelHeight: self labelHeight.
	self shouldIndicateKeyboardFocus ifTrue: [
		aCanvas
			roundRect: (0 @ 0 extent: extent x - borderWidth @ self labelHeight + borderWidth)
			color: (Theme current titlebarColorFromWindowFrameColor: widgetsColor)
			radius: Theme current roundedWindowRadius ].! !

!SystemWindow methodsFor: '*KeyboardFocus-testing' stamp: 'KLG 11/29/2021 12:46:56'!
shouldIndicateKeyboardFocus
	"Answer true if the titlebar should indicate the keyboard focus."

	^ Preferences titlebarIndicatesKeyboardFocus and: [
		self submorphHasKeyboardFocus ]! !

!SystemWindow methodsFor: '*KeyboardFocus-testing' stamp: 'KLG 11/29/2021 12:28:42'!
submorphHasKeyboardFocus
	"Answer true if one of my submoprhs have the keyboard focus."

	^ self runningWorld
		ifNotNil: [ :world |
			world activeHand keyboardFocus 
				ifNotNil: [ :morphInFocus |
					morphInFocus owningWindow == self ]
				ifNil: [ false ] ]
		ifNil: [ false ]! !

!Theme methodsFor: '*KeyboardFocus-colors' stamp: 'KLG 12/2/2021 14:33:01'!
titlebarColorFromWindowFrameColor: aWindowFrameColor
	"Convert the title bar color to the window frame color."

	^ aWindowFrameColor darker ! !

!DarkTheme methodsFor: '*KeyboardFocus-colors' stamp: 'KLG 12/2/2021 14:25:04'!
titlebarColorFromWindowFrameColor: aWindowFrameColor
	"Convert the title bar color to the window frame color."

	^ aWindowFrameColor mixed: 0.5 with: `Color orange`! !
