'From Cuis 6.0 [latest update: #5067] on 19 February 2022 at 4:32:15 pm'!
'Description Creates baselines of an image and code to check classes and methods conform to a baseline.'!
!provides: 'KlgBaseliner' 1 17!
SystemOrganization addCategory: 'KlgBaseliner'!


!classDefinition: #KlgBaseline category: 'KlgBaseliner'!
Object subclass: #KlgBaseline
	instanceVariableNames: 'imageVersion baselinesByClass baselinesByMetaclass classDefinitions metaclassDefinitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KlgBaseliner'!
!classDefinition: 'KlgBaseline class' category: 'KlgBaseliner'!
KlgBaseline class
	instanceVariableNames: ''!

!classDefinition: #KlgBaseliner category: 'KlgBaseliner'!
Object subclass: #KlgBaseliner
	instanceVariableNames: 'baselines lastBaselineCreated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KlgBaseliner'!
!classDefinition: 'KlgBaseliner class' category: 'KlgBaseliner'!
KlgBaseliner class
	instanceVariableNames: ''!


!KlgBaseline commentStamp: '<historical>' prior: 0!
I am a baseline of used by the base liner.!

!KlgBaseliner commentStamp: '<historical>' prior: 0!
I create a base line of all classes and their methods found in the image.!

!KlgBaseline methodsFor: 'iteration' stamp: 'KLG 10/31/2020 20:30:01'!
classesDo: aBlock
	"Evaluate aBlock with three arguments the class name, the class defintion and the dictionary timestamps."

	classDefinitions keysAndValuesDo: [ :className :definition |
		aBlock value: className value: definition value: (baselinesByClass at: className) ]! !

!KlgBaseline methodsFor: 'iteration' stamp: 'KLG 10/31/2020 20:29:40'!
metaclassesDo: aBlock
	"Evaluate aBlock with three arguments the class name, the class defintion and the dictionary timestamps."

	metaclassDefinitions keysAndValuesDo: [ :className :definition |
		aBlock value: className value: definition value: (baselinesByMetaclass at: className) ]! !

!KlgBaseline methodsFor: 'accessing' stamp: 'KLG 10/30/2020 11:13:35'!
imageVersion
	"Answer the value of imageVersion"

	self assert: imageVersion notNil.
	^ imageVersion! !

!KlgBaseline methodsFor: 'accessing' stamp: 'KLG 10/30/2020 11:13:35'!
imageVersion: anObject
	"Set the value of imageVersion"

	self assert: imageVersion isNil.
	imageVersion _ anObject! !

!KlgBaseline methodsFor: 'initialization' stamp: 'KLG 10/31/2020 20:22:53'!
initialize
	"Initialize the base line."

	super initialize.
	baselinesByClass _ IdentityDictionary new.
	baselinesByMetaclass _ IdentityDictionary new.
	metaclassDefinitions _ IdentityDictionary new.
	classDefinitions  _ IdentityDictionary new.! !

!KlgBaseline methodsFor: 'baselining' stamp: 'KLG 10/30/2020 20:12:20'!
baselineForClass: aClass
	"Create and answer a new baseline for aClass."

	^ self baselineForClassName: aClass name definition: aClass definition! !

!KlgBaseline methodsFor: 'baselining' stamp: 'KLG 10/31/2020 20:23:35'!
baselineForClassName: aClassName definition: aDefinition
	"Create and answer a new baseline for aClassName and aDefinition."

	classDefinitions at: aClassName put: aDefinition .
	^ baselinesByClass at: aClassName ifAbsentPut: [
		IdentityDictionary new ]! !

!KlgBaseline methodsFor: 'baselining' stamp: 'KLG 10/31/2020 20:24:47'!
baselineForMetaclass: aMetalass
	"Create and answer a new baseline for aMetalass."

	^ self baselineForMetaclassName: aMetalass theNonMetaClass name definition: aMetalass definition! !

!KlgBaseline methodsFor: 'baselining' stamp: 'KLG 10/31/2020 20:20:42'!
baselineForMetaclassName: aMetaclassName definition: aDefinition
	"Create and answer a new baseline for aMetaclassName and aDefinition."

	metaclassDefinitions at: aMetaclassName put: aDefinition .
	^ baselinesByMetaclass at: aMetaclassName ifAbsentPut: [
		IdentityDictionary new ]! !

!KlgBaseline methodsFor: 'checking' stamp: 'KLG 11/2/2020 10:45:13'!
dumpClassDefinitionCheckFor: aClass on: aStream
	"Dump code to check aClass defintion on aStream using me."
	
	| definition isMeta className |
	definition _ (isMeta _ aClass isMeta) 
		ifFalse: [ classDefinitions ]
		ifTrue: [ metaclassDefinitions ] :: 
			at: (className _ aClass theNonMetaClass name)
			ifAbsent: [ ^ self ].
	aStream
		newLine;
		nextPutAll: 'Smalltalk at: #';
		nextPutAll: className.
	isMeta ifTrue: [ aStream nextPutAll: ' class' ].
	aStream
		nextPutAll: ' ifPresent: [ :class | class definition = '.
	definition storeOn: aStream.
	aStream
		nextPutAll: ' ifFalse: [ self error: ''Class definition mismatch: ';
		nextPutAll: className;
		nextPutAll: '''] ].'
		
! !

!KlgBaseline methodsFor: 'checking' stamp: 'KLG 1/3/2021 20:12:52'!
dumpMethodTimeStampCheckFor: aMethod on: aStream
	"Dump code to check amethod check on aStream using me."
	
	| class baseline isMeta className timeStamp selector  |
	(class _ aMethod methodClass) ifNil: [ ^ self ]. ": Happens if class is not installed."
	baseline _ (isMeta _ class isMeta) 
		ifFalse: [ baselinesByClass ]
		ifTrue: [ baselinesByMetaclass ] :: at: (className _ class theNonMetaClass name).
	timeStamp _ baseline at: (selector _ aMethod selector) ifAbsent: [ ^ self ].
	aStream
		newLine;
		nextPutAll: 'Smalltalk at: #';
		nextPutAll: className.
	isMeta ifTrue: [ aStream nextPutAll: ' class' ].
	aStream
		nextPutAll: ' ifPresent: [ :class | class methodDict at: '.
	selector storeOn: aStream.
	aStream
		nextPutAll: '  ifPresent: [ :method | method timeStamp = '.
	timeStamp storeOn: aStream.
	aStream
		nextPutAll: ' ifFalse: [ self error: ''Timestamp mismatch: ';
		nextPutAll: className;
		nextPutAll: '>>#';
		nextPutAll: selector;
		nextPutAll: '''] ] ].'
		
! !

!KlgBaseline methodsFor: 'comparing' stamp: 'KLG 11/1/2020 22:45:40'!
= anOtherbaseline
	"Compare me to another base line."

	self imageVersion = anOtherbaseline imageVersion ifFalse: [ ^ false ].
	classDefinitions = (anOtherbaseline instVarNamed: #classDefinitions) ifFalse: [ ^ false ].
	metaclassDefinitions = (anOtherbaseline instVarNamed: #metaclassDefinitions) ifFalse: [ ^ false ].
	baselinesByClass = (anOtherbaseline instVarNamed: #baselinesByClass) ifFalse: [ ^ false ].
	baselinesByMetaclass = (anOtherbaseline instVarNamed: #baselinesByMetaclass) ifFalse: [ ^ false ].
	^ true
! !

!KlgBaseliner methodsFor: 'baselining' stamp: 'KLG 10/30/2020 16:49:10'!
baseline
	"Answer the base line for this image version"

	^ baselines at: SystemVersion current versionRevision ifAbsent: [
		self loadBaseline ]! !

!KlgBaseliner methodsFor: 'baselining' stamp: 'KLG 10/30/2020 16:50:31'!
createBaseline
	"Create a new base line object and file it."

	lastBaselineCreated _ self baselineClass new.
	self fillBaseline: lastBaselineCreated.
	^ baselines at: lastBaselineCreated imageVersion put: lastBaselineCreated! !

!KlgBaseliner methodsFor: 'baselining' stamp: 'KLG 11/1/2020 23:43:34'!
fillBaseline: aBaseline
	"Fill the base line."

	aBaseline imageVersion: SystemVersion current versionRevision	.
	Smalltalk allBehaviorsDo: [ :aBehavior | | behaviorBaseline |
		behaviorBaseline _ aBehavior isMeta
			ifFalse: [ aBaseline baselineForClass: aBehavior ]
			ifTrue: [ aBaseline baselineForMetaclass: aBehavior ].
		aBehavior methodsDo: [ :method |
			method timeStamp ifNotEmpty: [ :timeStamp |
				behaviorBaseline at: method selector put: timeStamp ] ] ]! !

!KlgBaseliner methodsFor: 'initialization' stamp: 'KLG 10/30/2020 16:49:28'!
initialize
	"Initialize the instance."

	super initialize.
	baselines _ Dictionary new! !

!KlgBaseliner methodsFor: 'instance creation' stamp: 'KLG 10/30/2020 11:09:54'!
baselineClass
	"Answer base line class to use."

	^ self class baselineClass! !

!KlgBaseliner methodsFor: 'filenames' stamp: 'KLG 10/30/2020 13:02:23'!
baselineFilename
	"Answer the baseline filename for this image version."

	^ self baselineFilenameFor: SystemVersion current versionRevision! !

!KlgBaseliner methodsFor: 'filenames' stamp: 'KLG 10/30/2020 13:02:29'!
baselineFilenameFor: aVersion
	"Answer the baseline filename for aVersion."

	^ 'Cuis-', ('-' join: (aVersion	 collect: [ :part | part asString ] )), '.baseline'! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 11/2/2020 18:17:47'!
checkForCodeFile: aCodeFile
	"Answer the code to check aClass defintion."

	^ String streamContents: [ :stream |
		self dumpCheckForCodeFile: aCodeFile onStream: stream ]
	! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 11/2/2020 18:20:18'!
dumpCheckForCodeFile: aCodeFile onFile: aFileEntryLike
	"Dump a check for aCodeFile's contents on a file."

	aFileEntryLike asFileEntry writeStreamDo: [ :stream |
		self dumpCheckForCodeFile: aCodeFile onStream: stream ]! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 2/19/2022 16:29:06'!
dumpCheckForCodeFile: aCodeFile onStream: aStream
	"Dump a check for aCodeFile's contents on aStream."

	aCodeFile classes do: [ :class |
		self dumpClassDefinitionCheckFor: class on: aStream ].
	aCodeFile allMethodReferences do: [ :method |
		self dumpMethodTimeStampCheckFor: method on: aStream ].
	aStream
		lf;
		nextPutAll: 'Preferences verboseBaseLineChecks ifTrue: [ self inform: ''Baseline check OK'' ].'! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 11/2/2020 18:21:24'!
dumpCheckOnFileForCodeFile: aCodeFile
	"Dump a check for aCodeFile's contents on a file."

	self dumpCheckForCodeFile: aCodeFile onFile: aCodeFile fullName, '.chk.st' ! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 11/2/2020 13:52:32'!
dumpClassDefinitionCheckFor: aClass
	"Answer the code to check aClass defintion."

	^ String streamContents: [ :stream |
		self dumpClassDefinitionCheckFor: aClass on: stream ]
	! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 11/1/2020 21:44:06'!
dumpClassDefinitionCheckFor: aClass on: aStream
	"Dump code to check aClass defintion on aStream. using the latest baseline."

	self baseline dumpClassDefinitionCheckFor: aClass on: aStream! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 11/1/2020 14:37:01'!
dumpMethodTimeStampCheckFor: aMethod on: aStream
	"Dump code to check amethod check on aStream using the latest baseline."

	self baseline dumpMethodTimeStampCheckFor: aMethod on: aStream! !

!KlgBaseliner methodsFor: 'checking' stamp: 'KLG 11/1/2020 14:40:21'!
methodTimeStampCheckFor: aMethod
	"Answer the method timeStamp check code for aMethod."

	^ String streamContents: [ :stream |
		self dumpMethodTimeStampCheckFor: aMethod on: stream ]! !

!KlgBaseliner methodsFor: 'persistence' stamp: 'KLG 10/30/2020 16:42:37'!
dumpBaseline
	"Persist the last base line drawn or loaded and answer it.
	
	If none exists create one."

	^ self dumpBaseline: (lastBaselineCreated ifNil: [ self createBaseline ])! !

!KlgBaseliner methodsFor: 'persistence' stamp: 'KLG 11/1/2020 22:12:08'!
dumpBaseline: aBaseline
	"Persist aBaseline."

	self baselineFilenameFor: aBaseline imageVersion ::
		asFileEntry writeStreamDo: [ :stream | 
			self dumpBaseline: aBaseline on: stream ].
	^ aBaseline! !

!KlgBaseliner methodsFor: 'persistence' stamp: 'KLG 11/1/2020 23:03:25'!
dumpBaseline: aBaseline on: aStream
	"Persist aBaseline."

	| dumpBlock |
	aStream 	nextChunkPut: self dumpFormatVersion.
	aBaseline imageVersion do: [ :part |
		aStream 	nextChunkPut: part asString].
	aStream nextChunkPut: '+++'.
	aBaseline		classesDo: (dumpBlock _ [ :className :definition :timestamps |
		aStream 
			nextChunkPut: className;
			nextChunkPut: definition.
		timestamps keysAndValuesDo: [ :selector :timestamp |
			aStream
				nextChunkPut: selector;
				nextChunkPut: timestamp ].
		aStream nextChunkPut: ']}{[' ] ).
	aStream nextChunkPut: '###'.
	aBaseline metaclassesDo: dumpBlock.
	aStream nextChunkPut: '###'.! !

!KlgBaseliner methodsFor: 'persistence' stamp: 'KLG 10/31/2020 20:08:05'!
dumpFormatVersion
	"Answer the version of the dump format."

	^ self class dumpFormatVersion! !

!KlgBaseliner methodsFor: 'persistence' stamp: 'KLG 11/1/2020 22:20:25'!
loadBaseline
	"Load a base line for this image version."

	^ self loadBaselineFromFile: self baselineFilename! !

!KlgBaseliner methodsFor: 'persistence' stamp: 'KLG 11/1/2020 22:20:41'!
loadBaselineFromFile: aFilename
	"Load the base line from aFilename.
	
	Answer it or nil, if loading fails."
	
	^ aFilename asFileEntry readStreamDo: [ :stream |
		self loadBaselineFromStream: stream ]
! !

!KlgBaseliner methodsFor: 'persistence' stamp: 'KLG 2/12/2021 16:22:30'!
loadBaselineFromStream: aStream
	"Load the base line from aStream"
	
	| newBaseline version chunk endCheck |
	newBaseline _ self baselineClass new.
	endCheck _ [ aStream atEnd ifTrue: [ self error: 'Premature end of file' ] ].
	self dumpFormatVersion = aStream nextChunk ifFalse: [ self error: 'Wrong baseline file version' ].
	endCheck value.
	version _ OrderedCollection new: 2.
	[ (chunk _ aStream nextChunk) = '+++' ] whileFalse: [
		version add: (chunk asNumber ifNil: [ ^ self error: 'Invalid version part: ', chunk ]).
		endCheck value ].
	version _ version asArray.
	newBaseline imageVersion: version.
	{[ :cN :d |		newBaseline baselineForClassName: cN definition: d ].
	  [ :cN :d | 		newBaseline baselineForMetaclassName: cN definition: d ]} do: [ :baselineGetter |
		[ (chunk _ aStream nextChunk) = '###' ] whileFalse: [ | className definition classBaseline |
				className _ chunk asSymbol.
				endCheck value.
				definition _ aStream nextChunk.
				classBaseline _ baselineGetter value: className value: definition.
				[ (chunk _ aStream nextChunk) = ']}{[' ] whileFalse: [ | selector timeStamp |
					selector _ chunk asSymbol.
					endCheck value.
					timeStamp _ aStream nextChunk.
					classBaseline at: selector put: timeStamp ] ] ].
	^ baselines at: version put: newBaseline! !

!KlgBaseliner class methodsFor: 'instance creation' stamp: 'KLG 10/30/2020 11:40:38'!
baselineClass
	"Answer the class for baselines."
	
	^ KlgBaseline! !

!KlgBaseliner class methodsFor: 'persistence' stamp: 'KLG 10/31/2020 20:26:03'!
dumpFormatVersion
	"Answer the version of the dump format."

	^ '1'! !

!KlgBaseliner class methodsFor: 'menu buildi	ng' stamp: 'KLG 11/3/2020 08:52:42'!
worldMenuOptions

	^{{
		#itemGroup 				-> 		50.
		#itemOrder 				-> 		10.
		#label 				-> 		'Create Baseline'.
		#object               -> 				         self.
		#selector 				-> 		#dumpBaseline.
		#balloonText				-> 		'Create a baseline to check a extension methods'.
	} asDictionary}! !

!KlgBaseliner class methodsFor: 'baselining' stamp: 'KLG 11/3/2020 08:55:12'!
dumpBaseline
	"Dump the base line for this image.
	
	Be carefull to use a clean image."
	self new dumpBaseline! !

!CodeFileBrowser methodsFor: '*KlgBaseliner' stamp: 'KLG 11/2/2020 18:40:49'!
writeCheckFile
	"Write a check file for the code file I browse."

	KlgBaseliner new dumpCheckOnFileForCodeFile: caseCodeSource! !

!CodeFileBrowserWindow methodsFor: '*KlgBaseliner' stamp: 'KLG 11/2/2020 18:37:10'!
codeFileListMenu
	| aMenu itemColl |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Code File'.
	itemColl _ OrderedCollection new.
	itemColl addAll:
		{
			{#label -> 'find class... (f)'. #selector -> #findClass} asDictionary. 
			nil
		}.
	self model baseCodeSource isLiveSmalltalkImage ifTrue: [
		itemColl add:
			{#label -> 'fileIn'. #object -> #model. #selector -> #fileIn} asDictionary ].
	itemColl add:
		{#label -> 'fileOut'. #object -> #model. #selector -> #fileOut} asDictionary.
	self model caseCodeSource isLiveSmalltalkImage ifFalse: [
		itemColl add:
			{#label -> 'remove existing'. #object -> #model. #selector -> #removeUnmodifiedClasses} asDictionary ].
	itemColl add:
		{ #label -> 'write check file'. #object -> #model. #selector -> #writeCheckFile} asDictionary.
	aMenu addItemsFromDictionaries: itemColl.
	^ aMenu.! !
